import torch
try:
    from plyfile import PlyData
    HAS_PLYFILE = True
except:
    HAS_PLYFILE = False
from torch_geometric.data import Data


def read_ply(path):
    if not HAS_PLYFILE:
        raise Exception("Reading ply files requires the `plyfile` package. Install it using `conda install -c conda-forge --no-deps plyfile` if you require this functionality.")
    with open(path, 'rb') as f:
        data = PlyData.read(f)

    pos = ([torch.tensor(data['vertex'][axis]) for axis in ['x', 'y', 'z']])
    pos = torch.stack(pos, dim=-1)

    face = None
    if 'face' in data:
        faces = data['face']['vertex_indices']
        faces = [torch.tensor(fa, dtype=torch.long) for fa in faces]
        face = torch.stack(faces, dim=-1)

    data = Data(pos=pos, face=face)

    return data
